@@ -1,5 +1,4 @@
 from .stringEnum import StringEnum
-from . import t
 
 dryRun = False
 errorLevel = ["fatal"]
@@ -8,8 +7,6 @@ quiet = True
 asciiOnly = False
 refStatus = StringEnum("current", "snapshot")
 biblioDisplay = StringEnum("index", "inline", "direct")
-specClass: t.Optional[t.Spec]
-specClass = None
 testAnnotationURL = "https://test.csswg.org/harness/annotate.js"
 chroot = True
 executeCode = False
